@@ -6,16 +6,16 @@
     Build epub files.
     Originally derived from qthelp.py.
 
-    :copyright: Copyright 2007-2015 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
 import os
 import re
-import time
 import codecs
 import zipfile
 from os import path
+from datetime import datetime
 
 try:
     from PIL import Image
@@ -40,9 +40,9 @@ from sphinx.util.console import brown
 # output but that may be customized by (re-)setting module attributes,
 # e.g. from conf.py.
 
-_mimetype_template = 'application/epub+zip'  # no EOL!
+MIMETYPE_TEMPLATE = 'application/epub+zip'  # no EOL!
 
-_container_template = u'''\
+CONTAINER_TEMPLATE = u'''\
 <?xml version="1.0" encoding="UTF-8"?>
 <container version="1.0"
       xmlns="urn:oasis:names:tc:opendocument:xmlns:container">
@@ -53,7 +53,7 @@ _container_template = u'''\
 </container>
 '''
 
-_toc_template = u'''\
+TOC_TEMPLATE = u'''\
 <?xml version="1.0"?>
 <ncx version="2005-1" xmlns="http://www.daisy.org/z3986/2005/ncx/">
   <head>
@@ -71,7 +71,7 @@ _toc_template = u'''\
 </ncx>
 '''
 
-_navpoint_template = u'''\
+NAVPOINT_TEMPLATE = u'''\
 %(indent)s  <navPoint id="%(navpoint)s" playOrder="%(playorder)d">
 %(indent)s    <navLabel>
 %(indent)s      <text>%(text)s</text>
@@ -79,10 +79,10 @@ _navpoint_template = u'''\
 %(indent)s    <content src="%(refuri)s" />
 %(indent)s  </navPoint>'''
 
-_navpoint_indent = '  '
-_navPoint_template = 'navPoint%d'
+NAVPOINT_INDENT = '  '
+NODE_NAVPOINT_TEMPLATE = 'navPoint%d'
 
-_content_template = u'''\
+CONTENT_TEMPLATE = u'''\
 <?xml version="1.0" encoding="UTF-8"?>
 <package xmlns="http://www.idpf.org/2007/opf" version="2.0"
       unique-identifier="%(uid)s">
@@ -109,41 +109,44 @@ _content_template = u'''\
 </package>
 '''
 
-_cover_template = u'''\
+COVER_TEMPLATE = u'''\
     <meta name="cover" content="%(cover)s"/>
 '''
 
-_coverpage_name = u'epub-cover.html'
+COVERPAGE_NAME = u'epub-cover.xhtml'
 
-_file_template = u'''\
+FILE_TEMPLATE = u'''\
     <item id="%(id)s"
           href="%(href)s"
           media-type="%(media_type)s" />'''
 
-_spine_template = u'''\
+SPINE_TEMPLATE = u'''\
     <itemref idref="%(idref)s" />'''
 
-_guide_template = u'''\
+GUIDE_TEMPLATE = u'''\
     <reference type="%(type)s" title="%(title)s" href="%(uri)s" />'''
 
-_toctree_template = u'toctree-l%d'
+TOCTREE_TEMPLATE = u'toctree-l%d'
 
-_link_target_template = u' [%(uri)s]'
+DOCTYPE = u'''<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN"
+  "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">'''
 
-_footnote_label_template = u'#%d'
+LINK_TARGET_TEMPLATE = u' [%(uri)s]'
 
-_footnotes_rubric_name = u'Footnotes'
+FOOTNOTE_LABEL_TEMPLATE = u'#%d'
 
-_css_link_target_class = u'link-target'
+FOOTNOTES_RUBRIC_NAME = u'Footnotes'
+
+CSS_LINK_TARGET_CLASS = u'link-target'
 
 # XXX These strings should be localized according to epub_language
-_guide_titles = {
+GUIDE_TITLES = {
     'toc': u'Table of Contents',
     'cover': u'Cover'
 }
 
-_media_types = {
-    '.html': 'application/xhtml+xml',
+MEDIA_TYPES = {
+    '.xhtml': 'application/xhtml+xml',
     '.css': 'text/css',
     '.png': 'image/png',
     '.gif': 'image/gif',
@@ -152,14 +155,15 @@ _media_types = {
     '.jpeg': 'image/jpeg',
     '.otf': 'application/x-font-otf',
     '.ttf': 'application/x-font-ttf',
+    '.woff': 'application/font-woff',
 }
 
-_vector_graphics_extensions = ('.svg',)
+VECTOR_GRAPHICS_EXTENSIONS = ('.svg',)
 
 # Regular expression to match colons only in local fragment identifiers.
 # If the URI contains a colon before the #,
 # it is an external link that should not change.
-_refuri_re = re.compile("([^#:]*#)(.*)")
+REFURI_RE = re.compile("([^#:]*#)(.*)")
 
 
 # The epub publisher
@@ -184,10 +188,34 @@ class EpubBuilder(StandaloneHTMLBuilder):
     # don't add sidebar etc.
     embedded = True
 
+    # don't generate search index or include search page
+    search = False
+
+    mimetype_template = MIMETYPE_TEMPLATE
+    container_template = CONTAINER_TEMPLATE
+    toc_template = TOC_TEMPLATE
+    navpoint_template = NAVPOINT_TEMPLATE
+    navpoint_indent = NAVPOINT_INDENT
+    node_navpoint_template = NODE_NAVPOINT_TEMPLATE
+    content_template = CONTENT_TEMPLATE
+    cover_template = COVER_TEMPLATE
+    coverpage_name = COVERPAGE_NAME
+    file_template = FILE_TEMPLATE
+    spine_template = SPINE_TEMPLATE
+    guide_template = GUIDE_TEMPLATE
+    toctree_template = TOCTREE_TEMPLATE
+    doctype = DOCTYPE
+    link_target_template = LINK_TARGET_TEMPLATE
+    css_link_target_class = CSS_LINK_TARGET_CLASS
+    guide_titles = GUIDE_TITLES
+    media_types = MEDIA_TYPES
+    refuri_re = REFURI_RE
+
     def init(self):
         StandaloneHTMLBuilder.init(self)
         # the output files for epub must be .html only
-        self.out_suffix = '.html'
+        self.out_suffix = '.xhtml'
+        self.link_suffix = '.xhtml'
         self.playorder = 0
         self.tocid = 0
 
@@ -225,7 +253,7 @@ class EpubBuilder(StandaloneHTMLBuilder):
                 return result
             classes = doctree.parent.attributes['classes']
             for level in range(8, 0, -1):  # or range(1, 8)?
-                if (_toctree_template % level) in classes:
+                if (self.toctree_template % level) in classes:
                     result.append({
                         'level': level,
                         'refuri': self.esc(refuri),
@@ -257,7 +285,7 @@ class EpubBuilder(StandaloneHTMLBuilder):
         """
         refnodes.insert(0, {
             'level': 1,
-            'refuri': self.esc(self.config.master_doc + '.html'),
+            'refuri': self.esc(self.config.master_doc + self.out_suffix),
             'text': ssp(self.esc(
                 self.env.titles[self.config.master_doc].astext()))
         })
@@ -286,7 +314,7 @@ class EpubBuilder(StandaloneHTMLBuilder):
         """
         for node in tree.traverse(nodes.reference):
             if 'refuri' in node:
-                m = _refuri_re.match(node['refuri'])
+                m = self.refuri_re.match(node['refuri'])
                 if m:
                     node['refuri'] = self.fix_fragment(m.group(1), m.group(2))
             if 'refid' in node:
@@ -332,11 +360,11 @@ class EpubBuilder(StandaloneHTMLBuilder):
                 return fn.parent, fn.parent.index(fn) + 1
             for node in tree.traverse(nodes.rubric):
                 if len(node.children) == 1 and \
-                        node.children[0].astext() == _footnotes_rubric_name:
+                        node.children[0].astext() == FOOTNOTES_RUBRIC_NAME:
                     return node.parent, node.parent.index(node) + 1
             doc = tree.traverse(nodes.document)[0]
             rub = nodes.rubric()
-            rub.append(nodes.Text(_footnotes_rubric_name))
+            rub.append(nodes.Text(FOOTNOTES_RUBRIC_NAME))
             doc.append(rub)
             return doc, doc.index(rub) + 1
 
@@ -352,12 +380,12 @@ class EpubBuilder(StandaloneHTMLBuilder):
                     uri.startswith('ftp:')) and uri not in node.astext():
                 idx = node.parent.index(node) + 1
                 if show_urls == 'inline':
-                    uri = _link_target_template % {'uri': uri}
+                    uri = self.link_target_template % {'uri': uri}
                     link = nodes.inline(uri, uri)
-                    link['classes'].append(_css_link_target_class)
+                    link['classes'].append(self.css_link_target_class)
                     node.parent.insert(idx, link)
                 elif show_urls == 'footnote':
-                    label = _footnote_label_template % nr
+                    label = FOOTNOTE_LABEL_TEMPLATE % nr
                     nr += 1
                     footnote_ref = make_footnote_ref(doc, label)
                     node.parent.insert(idx, footnote_ref)
@@ -382,15 +410,15 @@ class EpubBuilder(StandaloneHTMLBuilder):
         # XXX: modifies tree inline
         # Logic modeled from themes/basic/genindex.html
         for key, columns in tree:
-            for entryname, (links, subitems) in columns:
+            for entryname, (links, subitems, key_) in columns:
                 for (i, (ismain, link)) in enumerate(links):
-                    m = _refuri_re.match(link)
+                    m = self.refuri_re.match(link)
                     if m:
                         links[i] = (ismain,
                                     self.fix_fragment(m.group(1), m.group(2)))
                 for subentryname, subentrylinks in subitems:
                     for (i, (ismain, link)) in enumerate(subentrylinks):
-                        m = _refuri_re.match(link)
+                        m = self.refuri_re.match(link)
                         if m:
                             subentrylinks[i] = (ismain,
                                                 self.fix_fragment(m.group(1), m.group(2)))
@@ -398,7 +426,7 @@ class EpubBuilder(StandaloneHTMLBuilder):
     def is_vector_graphics(self, filename):
         """Does the filename extension indicate a vector graphic format?"""
         ext = path.splitext(filename)[-1]
-        return ext in _vector_graphics_extensions
+        return ext in VECTOR_GRAPHICS_EXTENSIONS
 
     def copy_image_files_pil(self):
         """Copy images using the PIL.
@@ -461,6 +489,7 @@ class EpubBuilder(StandaloneHTMLBuilder):
         """
         if pagename.startswith('genindex'):
             self.fix_genindex(addctx['genindexentries'])
+        addctx['doctype'] = self.doctype
         StandaloneHTMLBuilder.handle_page(self, pagename, addctx, templatename,
                                           outfilename, event_arg)
 
@@ -479,7 +508,7 @@ class EpubBuilder(StandaloneHTMLBuilder):
         self.info('writing %s file...' % outname)
         f = codecs.open(path.join(outdir, outname), 'w', 'utf-8')
         try:
-            f.write(_mimetype_template)
+            f.write(self.mimetype_template)
         finally:
             f.close()
 
@@ -494,7 +523,7 @@ class EpubBuilder(StandaloneHTMLBuilder):
                 raise
         f = codecs.open(path.join(outdir, outname), 'w', 'utf-8')
         try:
-            f.write(_container_template)
+            f.write(self.container_template)
         finally:
             f.close()
 
@@ -511,7 +540,7 @@ class EpubBuilder(StandaloneHTMLBuilder):
         metadata['copyright'] = self.esc(self.config.epub_copyright)
         metadata['scheme'] = self.esc(self.config.epub_scheme)
         metadata['id'] = self.esc(self.config.epub_identifier)
-        metadata['date'] = self.esc(time.strftime('%Y-%m-%d'))
+        metadata['date'] = self.esc(datetime.utcnow().strftime("%Y-%m-%d"))
         metadata['files'] = files
         metadata['spine'] = spine
         metadata['guide'] = guide
@@ -539,17 +568,17 @@ class EpubBuilder(StandaloneHTMLBuilder):
                 if filename in self.ignored_files:
                     continue
                 ext = path.splitext(filename)[-1]
-                if ext not in _media_types:
+                if ext not in self.media_types:
                     # we always have JS and potentially OpenSearch files, don't
                     # always warn about them
                     if ext not in ('.js', '.xml'):
                         self.warn('unknown mimetype for %s, ignoring' % filename)
                     continue
                 filename = filename.replace(os.sep, '/')
-                projectfiles.append(_file_template % {
+                projectfiles.append(self.file_template % {
                     'href': self.esc(filename),
                     'id': self.esc(self.make_id(filename)),
-                    'media_type': self.esc(_media_types[ext])
+                    'media_type': self.esc(self.media_types[ext])
                 })
                 self.files.append(filename)
 
@@ -560,20 +589,20 @@ class EpubBuilder(StandaloneHTMLBuilder):
                 continue
             if item['refuri'] in self.ignored_files:
                 continue
-            spine.append(_spine_template % {
+            spine.append(self.spine_template % {
                 'idref': self.esc(self.make_id(item['refuri']))
             })
         for info in self.domain_indices:
-            spine.append(_spine_template % {
+            spine.append(self.spine_template % {
                 'idref': self.esc(self.make_id(info[0] + self.out_suffix))
             })
         if self.get_builder_config('use_index', 'epub'):
-            spine.append(_spine_template % {
+            spine.append(self.spine_template % {
                 'idref': self.esc(self.make_id('genindex' + self.out_suffix))
             })
 
         # add the optional cover
-        content_tmpl = _content_template
+        content_tmpl = self.content_template
         html_tmpl = None
         if self.config.epub_cover:
             image, html_tmpl = self.config.epub_cover
@@ -581,22 +610,22 @@ class EpubBuilder(StandaloneHTMLBuilder):
             mpos = content_tmpl.rfind('</metadata>')
             cpos = content_tmpl.rfind('\n', 0, mpos) + 1
             content_tmpl = content_tmpl[:cpos] + \
-                _cover_template % {'cover': self.esc(self.make_id(image))} + \
+                COVER_TEMPLATE % {'cover': self.esc(self.make_id(image))} + \
                 content_tmpl[cpos:]
             if html_tmpl:
-                spine.insert(0, _spine_template % {
-                    'idref': self.esc(self.make_id(_coverpage_name))})
-                if _coverpage_name not in self.files:
-                    ext = path.splitext(_coverpage_name)[-1]
-                    self.files.append(_coverpage_name)
-                    projectfiles.append(_file_template % {
-                        'href': self.esc(_coverpage_name),
-                        'id': self.esc(self.make_id(_coverpage_name)),
-                        'media_type': self.esc(_media_types[ext])
+                spine.insert(0, self.spine_template % {
+                    'idref': self.esc(self.make_id(self.coverpage_name))})
+                if self.coverpage_name not in self.files:
+                    ext = path.splitext(self.coverpage_name)[-1]
+                    self.files.append(self.coverpage_name)
+                    projectfiles.append(self.file_template % {
+                        'href': self.esc(self.coverpage_name),
+                        'id': self.esc(self.make_id(self.coverpage_name)),
+                        'media_type': self.esc(self.media_types[ext])
                     })
                 ctx = {'image': self.esc(image), 'title': self.config.project}
                 self.handle_page(
-                    path.splitext(_coverpage_name)[0], ctx, html_tmpl)
+                    path.splitext(self.coverpage_name)[0], ctx, html_tmpl)
 
         guide = []
         auto_add_cover = True
@@ -610,21 +639,21 @@ class EpubBuilder(StandaloneHTMLBuilder):
                     auto_add_cover = False
                 if type == 'toc':
                     auto_add_toc = False
-                guide.append(_guide_template % {
+                guide.append(self.guide_template % {
                     'type': self.esc(type),
                     'title': self.esc(title),
                     'uri': self.esc(uri)
                 })
         if auto_add_cover and html_tmpl:
-            guide.append(_guide_template % {
+            guide.append(self.guide_template % {
                 'type': 'cover',
-                'title': _guide_titles['cover'],
-                'uri': self.esc(_coverpage_name)
+                'title': self.guide_titles['cover'],
+                'uri': self.esc(self.coverpage_name)
             })
         if auto_add_toc and self.refnodes:
-            guide.append(_guide_template % {
+            guide.append(self.guide_template % {
                 'type': 'toc',
-                'title': _guide_titles['toc'],
+                'title': self.guide_titles['toc'],
                 'uri': self.esc(self.refnodes[0]['refuri'])
             })
         projectfiles = '\n'.join(projectfiles)
@@ -645,10 +674,10 @@ class EpubBuilder(StandaloneHTMLBuilder):
         if incr:
             self.playorder += 1
         self.tocid += 1
-        node['indent'] = _navpoint_indent * level
-        node['navpoint'] = self.esc(_navPoint_template % self.tocid)
+        node['indent'] = self.navpoint_indent * level
+        node['navpoint'] = self.esc(self.node_navpoint_template % self.tocid)
         node['playorder'] = self.playorder
-        return _navpoint_template % node
+        return self.navpoint_template % node
 
     def insert_subnav(self, node, subnav):
         """Insert nested navpoints for given node.
@@ -731,7 +760,7 @@ class EpubBuilder(StandaloneHTMLBuilder):
         level = min(level, self.config.epub_tocdepth)
         f = codecs.open(path.join(outdir, outname), 'w', 'utf-8')
         try:
-            f.write(_toc_template % self.toc_metadata(level, navpoints))
+            f.write(self.toc_template % self.toc_metadata(level, navpoints))
         finally:
             f.close()
 
